/*==========================================================================*\
|                                                                            |
| LowLevelFunc.c                                                             |
|                                                                            |
| Low Level Functions regarding user's Hardware                              |
|----------------------------------------------------------------------------|
| Project:              MSP430 Replicator                                    |
| Developed using:      IAR Embedded Workbench 3.40B [Kickstart]             |
|----------------------------------------------------------------------------|
| Author:               FRGR                                                 |
| Version:              1.5                                                  |
| Initial Version:      04-17-02                                             |
| Last Change:          09-21-05                                             |
|----------------------------------------------------------------------------|
| Version history:                                                           |
| 1.0 04/02 FRGR        Initial version.                                     |
| 1.1 04/02 FRGR        Included SPI mode to speed up shifting function by 2.|
| 1.2 06/02 ALB2        Formatting changes, added comments.                  |
| 1.3 08/02 ALB2        Initial code release with Lit# SLAA149.              |
| 1.4 09/05 SUN1        Software delays redesigned to use TimerA harware;    |
|                       see MsDelay() routine. Added TA setup                |
| 1.5 12/05 STO         Adapted for 2xx devices with SpyBiWire using 4JTAG   |
|----------------------------------------------------------------------------|
| Designed 2002 by Texas Instruments Germany                                 |
\*==========================================================================*/
#include <windows.h>
#include <stdio.h>
#include "LowLevelFunc.h"
#include "ftd2xx.h"


//#define MAXBAURD        FT_BAUD_921600      /* őްڰ */
#define MAXBUFF         (63448)             /* by AN232B-03_D2XXDataThroughput.pdf */
//#define MAXBUFF         (65536)             /* by AN232B-03_D2XXDataThroughput.pdf */

extern unsigned char  gbLatencyTimer;
extern unsigned long  gnBaud;

unsigned char  gbPrevLatencyTimer;

outPortType out_word;


//FILE *gfpin;   /* Legacy DEBUG_MODE file pointer */

FT_HANDLE  ghFT = 0;


int     gnTxd;
//LPBYTE  gbTxd;
BYTE  gbTxd[MAXBUFF];

DWORD   gdwTck;
DWORD   gdwTdo;


/*====================================================================
  _FlushTxBuf
  ====================================================================*/
static VOID WINAPI FlushTxBuf(VOID) {
    DWORD  cbWrite;

    if(gnTxd > 0) {
        FT_Write(ghFT,gbTxd,gnTxd,&cbWrite);
        gnTxd = 0;
    }
}
/*====================================================================
  _UpdateTxBuf
  ====================================================================*/
VOID WINAPI UpdateTxBuf(BYTE txd) {
    DWORD  cbWrite;

gdwTck++;

    gbTxd[gnTxd++] = txd;
    if(gnTxd >= MAXBUFF) {
        FT_Write(ghFT,gbTxd,gnTxd,&cbWrite);
        gnTxd = 0;
    }
}
/*====================================================================
  _OpenComm
  ====================================================================*/
static BOOL WINAPI OpenComm(VOID) {
    if(FT_OK != FT_Open(0,&ghFT)) {
        printf("open error\n");
        return(FALSE);
    }
//    FT_SetBaudRate(ghFT,MAXBAURD); //9600~16=153600޲/sec
    FT_SetBaudRate(ghFT,gnBaud); //9600~16=153600޲/sec
    FT_SetBitMode(ghFT,0x0B,1);//0:input,1:output,Asynchronous Bit Bang
    if(gbLatencyTimer) {
        FT_GetLatencyTimer(ghFT,&gbPrevLatencyTimer);
        FT_SetLatencyTimer(ghFT,gbLatencyTimer);
    }
    gnTxd = 0;
    return(TRUE);
}
/*====================================================================
  _CloseComm
  ====================================================================*/
static BOOL WINAPI CloseComm(VOID) {
    if(ghFT) {
        FlushTxBuf();                             // add
        MsDelay(50);
        if(gbLatencyTimer) {
            FT_SetLatencyTimer(ghFT,gbPrevLatencyTimer);
        }
        FT_Close(ghFT);
        ghFT = (FT_HANDLE)NULL;

printf("gdwTck=%d, gdwTdo=%d\n",gdwTck,gdwTdo);

        return(TRUE);
    }
    return(FALSE);
}


/*====================================================================
  _readTDOBit
  ====================================================================*/
/* read the TDO bit from port */
unsigned char WINAPI readTDOBit(void) {
    BYTE   brxd;

    FlushTxBuf();

//    Sleep(1);

gdwTdo++;

    FT_GetBitMode(ghFT,&brxd);
    return((brxd & 0x04)?  1 : 0);
}

/****************************************************************************/
/* Function declarations which have to be programmed by the user for use    */
/* with hosts other than the MSP430F149.                                    */
/*                                                                          */
/* The following MSP430F149-specific code can be used as a reference as to  */
/* how to implement the required JTAG communication on additional hosts.    */
/****************************************************************************/

/*----------------------------------------------------------------------------
   Initialization of the Controller Board (Master or Host)
*/
void InitController(void) {
}

/*----------------------------------------------------------------------------
   This function switches TDO to Input, used for fuse blowing
*/
void TDOisInput(void) {
}

/*----------------------------------------------------------------------------
   Initialization of the Target Board (switch voltages on, preset JTAG pins)
    for devices with normal 4wires JTAG  (JTAG4SBW=0)
    for devices with Spy-Bi-Wire to work in 4wires JTAG (JTAG4SBW=1)
*/
int InitTarget(void) {
    if(OpenComm()) {
        SetRST();
        SetTMS();
        SetTDI();
        SetTCLK();
        SetTCK();
        FlushTxBuf();
        MsDelay(50);                // Settle MOS relays, target capacitor
        return(TRUE);
    }
    return(FALSE);
}

/*----------------------------------------------------------------------------
   Release Target Board (switch voltages off, JTAG pins are HI-Z)
*/
void ReleaseTarget(void) {
//ɃrbgEoOE[ĥscñvAbv֎~_ǉv!!!
//    FT_SetBitMode(ghFT,0x0F,1);//0:input,1:output,Asynchronous Bit Bang
//    UpdateTxBuf(0);
//    FlushTxBuf();
//    MsDelay(5);                // Settle MOS relays

    CloseComm();
    MsDelay(5);                // Settle MOS relays
}

//----------------------------------------------------------------------------
/*  Shift a value into TDI (MSB first) and simultaneously shift out a value
    from TDO (MSB first).
    Note:      When defining SPI_MODE the embedded SPI is used to speed up by 2.
    Arguments: word Format (number of bits shifted, 8 (F_BYTE) or 16 (F_WORD))
               word Data (data to be shifted into TDI)
    Result:    word (scanned TDO value)
*/
word ShiftX(word Format,word Data) {
    word tclk = StoreTCLK();        // Store TCLK state;
    word TDOword = 0x0000;            // Initialize shifted-in word
    word MSB = 0x0000;
    volatile word i;

    (Format == F_WORD) ? (MSB = 0x8000) : (MSB = 0x80);
    for(i = Format;i > 0;i--) {
        ((Data & MSB) == 0) ? ClrTDI() : SetTDI();
        Data <<= 1;
        if(i == 1)                    // Last bit requires TMS=1
           SetTMS();
        ClrTCK();
        SetTCK();
        TDOword <<= 1;                // TDO could be any port pin
        if(ScanTDO() != 0)
            TDOword++;
    }

    // common exit
    RestoreTCLK(tclk);              // restore TCLK state

    // JTAG FSM = Exit-DR
    ClrTCK();
    SetTCK();
    // JTAG FSM = Update-DR
    ClrTMS();
    ClrTCK();
    SetTCK();
//    FlushTxBuf();
    // JTAG FSM = Run-Test/Idle
    return(TDOword);
}

//----------------------------------------------------------------------------
/*  Shift a value into TDI (MSB first) and simultaneously shift out a value
    from TDO (MSB first).
    Note:      When defining SPI_MODE the embedded SPI is used to speed up by 2.
    Arguments: word Format (number of bits shifted, 8 (F_BYTE) or 16 (F_WORD))
               word Data (data to be shifted into TDI)
    Result:    word (scanned TDO value)
*/
void Shift(word Format,word Data) {
    word tclk = StoreTCLK();        // Store TCLK state;
    word MSB = 0x0000;
    volatile word i;

    (Format == F_WORD) ? (MSB = 0x8000) : (MSB = 0x80);
    for(i = Format;i > 0;i--) {
        ((Data & MSB) == 0) ? ClrTDI() : SetTDI();
        Data <<= 1;
        if(i == 1)                    // Last bit requires TMS=1
           SetTMS();
        ClrTCK();
        SetTCK();
    }

    // common exit
    RestoreTCLK(tclk);              // restore TCLK state

    // JTAG FSM = Exit-DR
    ClrTCK();
    SetTCK();
    // JTAG FSM = Update-DR
    ClrTMS();
    ClrTCK();
    SetTCK();
//    FlushTxBuf();
    // JTAG FSM = Run-Test/Idle
}

/*---------------------------------------------------------------------------
   Delay function (resolution is 1 ms)
   Arguments: word millisec (number of ms, max number is 0xFFFF)
*/
void MsDelay(word milliseconds) {
   Sleep(milliseconds);
}

/*---------------------------------------------------------------------------
   Delay function (resolution is ~1 us at 8MHz clock)
   Arguments: word microeconds (number of ms, max number is 0xFFFF)
*/
void usDelay(word microeconds) {
    do {
//        _NOP();
//        _NOP();
    } while(--microeconds > 0);
}

/*---------------------------------------------------------------------------
   This function generates Amount strobes with the Flash Timing Generator
   Frequency fFTG = 257..476kHz (t = 3.9..2.1us).
   User knows target frequency, instruction cycles, C implementation.
   Arguments: word Amount (number of strobes to be generated)
*/
#define S_LOOPBODY        14        // 14 cycles/loop w/o NOPs
#define S_ADDNOPS    (word)((FREQUENCY * 2.1) / 1000 - S_LOOPBODY + 1)
                        // S_ADDNOPS = 3..18
void TCLKstrobes(word Amount) {
    volatile word i;

    for(i = Amount;i > 0;i--) {       // This implementation has 14 body cycles!
        SetTCLK();
//        _NOP();                    // Include NOPs if necessary (min. 3)
//        _NOP();
//        _NOP();
        ClrTCLK();
    }
}

/*----------------------------------------------------------------------------
   This function controls the status LEDs depending on the status
   argument. It stops program in error case.
   Arguments: word status (4 stati possible for 2 LEDs)
              word index (additional number for detailed diagnostics or
                          watch variable during debugging phase)
*/
void ShowStatus(word status,word index) {
    printf("status = %d, index = %d\n",status,index);
}                                        // return if active, idle, ok

/****************************************************************************/
/*                         END OF SOURCE FILE                               */
/****************************************************************************/
