/*==========================================================================*\
|                                                                            |
| LowLevelFunc.c                                                             |
|                                                                            |
| Low Level Functions regarding user's Hardware                              |
|----------------------------------------------------------------------------|
| Project:              MSP430 Replicator                                    |
| Developed using:      IAR Embedded Workbench 3.40B [Kickstart]             |
|----------------------------------------------------------------------------|
| Author:               FRGR                                                 |
| Version:              1.5                                                  |
| Initial Version:      04-17-02                                             |
| Last Change:          09-21-05                                             |
|----------------------------------------------------------------------------|
| Version history:                                                           |
| 1.0 04/02 FRGR        Initial version.                                     |
| 1.1 04/02 FRGR        Included SPI mode to speed up shifting function by 2.|
| 1.2 06/02 ALB2        Formatting changes, added comments.                  |
| 1.3 08/02 ALB2        Initial code release with Lit# SLAA149.              |
| 1.4 09/05 SUN1        Software delays redesigned to use TimerA harware;    |
|                       see MsDelay() routine. Added TA setup                |
| 1.5 12/05 STO         Adapted for 2xx devices with SpyBiWire using 4JTAG   |
|----------------------------------------------------------------------------|
| Designed 2002 by Texas Instruments Germany                                 |
\*==========================================================================*/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "LowLevelFunc.h"
#include "ftd2xx.h"


//#define MAXBAURD        FT_BAUD_921600      /* őްڰ */
#define MAXBUFF         (63448)             /* by AN232B-03_D2XXDataThroughput.pdf */

unsigned long  gnBaud = FT_BAUD_921600;
unsigned char  gbLatencyTimer = 0;
int            gope = 0;
unsigned char  gbPrevLatencyTimer;

outPortType out_word;


FT_HANDLE  ghFT = 0;

int     gnTxd;
BYTE    gbTxd[MAXBUFF];
DWORD   gdwTck;
DWORD   gdwTdo;


/*--------------------------------------------------------------------*/
/* [֐] Mޯ̧ׯ                                           */
/* [  ] VOID WINAPI FlushTxBuf(VOID)                              */
/* [  ] none                                                      */
/* [o  ] none                                                      */
/* [T  v] Mޯ̧ްꍇASďo͂                   */
/* [Q  ]                                                           */
/* [  ]                                                           */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* [  ]  e                                    DateFcoding     */
/*--------------------------------------------------------------------*/
static VOID WINAPI FlushTxBuf(VOID) {
    DWORD  cbWrite;

    if(gnTxd > 0) {
        FT_Write(ghFT,gbTxd,gnTxd,&cbWrite);
        gnTxd = 0;
    }
}
/*--------------------------------------------------------------------*/
/* [֐] Mްݒ                                              */
/* [  ] VOID WINAPI UpdateTxBuf(BYTE txd)                         */
/* [  ] txd:Mް                                              */
/* [o  ] none                                                      */
/* [T  v] Mްޯ̧ɕۑBޒ߂őM             */
/* [Q  ]                                                           */
/* [  ]                                                           */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* [  ]  e                                    DateFcoding     */
/*--------------------------------------------------------------------*/
VOID WINAPI UpdateTxBuf(BYTE txd) {
    DWORD  cbWrite;

gdwTck++;

    gbTxd[gnTxd++] = txd;
    if(gnTxd >= MAXBUFF) {
        FT_Write(ghFT,gbTxd,gnTxd,&cbWrite);
        gnTxd = 0;
    }
}
/*--------------------------------------------------------------------*/
/* [֐] ٥                                               */
/* [  ] BOOL WINAPI OpenComm(VOID)                                */
/* [  ] none                                                      */
/* [o  ] ʺ(FALSE:sTRUE:)                            */
/* [T  v] FTDI޲݂                                     */
/* [Q  ]                                                           */
/* [  ]                                                           */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* [  ]  e                                    DateFcoding     */
/*--------------------------------------------------------------------*/
static BOOL WINAPI OpenComm(VOID) {
    if(FT_OK != FT_Open(0,&ghFT)) {
        printf("open error\n");
        return(FALSE);
    }
#if 0
//    FT_SetBaudRate(ghFT,MAXBAURD); //9600~16=153600޲/sec
    FT_SetBaudRate(ghFT,gnBaud); //9600~16=153600޲/sec
    FT_SetBitMode(ghFT,0xAC,1);//0:input,1:output,Asynchronous Bit Bang
#else
    if(FT_OK != FT_SetBaudRate(ghFT,gnBaud)) { //9600~16=153600޲/sec
        printf("FT_SetBaudRate error\n");
        return(FALSE);
    }
    if(FT_OK != FT_SetBitMode(ghFT,0xAC,1)) {//0:input,1:output,Asynchronous Bit Bang
        printf("FT_SetBitMode error\n");
        return(FALSE);
    }
#endif
    if(gbLatencyTimer) {
        FT_GetLatencyTimer(ghFT,&gbPrevLatencyTimer);
        FT_SetLatencyTimer(ghFT,gbLatencyTimer);
    }
    gnTxd = 0;
    return(TRUE);
}
/*--------------------------------------------------------------------*/
/* [֐] ٥۰                                               */
/* [  ] BOOL WINAPI closeComm(VOID)                               */
/* [  ] none                                                      */
/* [o  ] ʺ(FALSE:sTRUE:)                            */
/* [T  v] FTDI޲۰ނ                                     */
/* [Q  ]                                                           */
/* [  ]                                                           */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* [  ]  e                                    DateFcoding     */
/*--------------------------------------------------------------------*/
static BOOL WINAPI CloseComm(VOID) {
    if(ghFT) {
        FlushTxBuf();                             // add
        Sleep(50);
        if(gbLatencyTimer) {
            FT_SetLatencyTimer(ghFT,gbPrevLatencyTimer);
        }
        FT_Close(ghFT);
        ghFT = (FT_HANDLE)NULL;

printf("gdwTck=%d, gdwTdo=%d\n",gdwTck,gdwTdo);

        return(TRUE);
    }
    return(FALSE);
}
/*--------------------------------------------------------------------*/
/* [֐] ߰ēǂݏo                                              */
/* [  ] unsigned char WINAPI readTDOBit(void)                     */
/* [  ] none                                                      */
/* [o  ] ǂݏo߰Ēl                                            */
/* [T  v] Mޯ̧ްꍇSďo͂A߰Ăǂݏo       */
/* [Q  ]                                                           */
/* [  ]                                                           */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* [  ]  e                                    DateFcoding     */
/*--------------------------------------------------------------------*/
unsigned char WINAPI readTDOBit(void) {
    BYTE   brxd;

    FlushTxBuf();

//    Sleep(1);

gdwTdo++;

    FT_GetBitMode(ghFT,&brxd);
    return((brxd & 0x04)?  1 : 0);
}



/*--------------------------------------------------------------------*/
/* [֐] decoopt                                                   */
/* [  ] int decoopt(int argc,char **argv,FILE **lppifp)           */
/* [  ] argc   : argv̗L                                     */
/*          argv   : z߲                                  */
/* [o  ] ߼݉͌(=0:I!=0:ُI)                  */
/* [T  v] ޥײ݂Ƃēnꂽ߼݂͂                   */
/* [Q  ]                                                           */
/* [  ]                                                           */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* [  ]  e                                    DateFcoding     */
/*--------------------------------------------------------------------*/
int decoopt(int argc,char **argv) {
    int    nloc;
    int    rtc = 0;

    for(nloc = 1;nloc < argc;nloc++) {
        if((argv[nloc][0] == '-')||
           (argv[nloc][0] == '/')) {
            switch(argv[nloc][1] & 0xDF) {
                case 'B':
                    gnBaud = (unsigned long)atoi(&argv[nloc][2]);
                    break;
                case 'T':
                    gbLatencyTimer = (unsigned char)atoi(&argv[nloc][2]);
                    break;
                case 'O':
                    gope = atoi(&argv[nloc][2]);
                    break;
                default:
                    rtc = -1;
                    fprintf(stderr,"`ĂȂ߼݂ݒ肳Ă܂B\n");
            }
        }
    }
    return(rtc);
}
/*--------------------------------------------------------------------*/
/* [֐] main                                                      */
/* [  ] int main(int argc,char **argv)                            */
/* [  ] argc   : argv̗L                                     */
/*          argv   : z߲                                  */
/* [o  ]                                                        */
/* [T  v] Ҳ݊֐                                                   */
/* [Q  ]                                                           */
/* [  ]                                                           */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* [  ]  e                                    DateFcoding     */
/*--------------------------------------------------------------------*/
int main(int argc,char **argv) {
    DWORD  dwStart;
    int    nc;
    unsigned char  txd = 0xAC;

    do {
        /* ُ͈̐ */
        if(decoopt(argc,argv) != 0) {
            printf("usage: %s [-on] [-bn] [-tn]\n",argv[0]);
            printf("            |     |     +--- Latency Timer value (2-255)\n");
            printf("            |     +--------- Baudrate (300-9216000)\n");
            printf("            +--------------- operation (0:write, 1:read)\n");
            break;
        }
        if(!OpenComm()) {
            break;
        }
        if(gope) {
            while(1) {
                if(_kbhit()) {
                    if(getch() == '\033') {
                        break;
                    }
                }
                for(dwStart = GetTickCount(),nc = 0;nc < 1000;nc++) {
                    readTDOBit();
                }
                printf("past time = %d\n",GetTickCount() - dwStart);
            }
        } else {
            while(1) {
                if(_kbhit()) {
                    if(getch() == '\033') {
                        break;
                    }
                }
                for(dwStart = GetTickCount(),nc = 0;nc < 10000;nc++) {
                    UpdateTxBuf(txd);
                    txd ^= 0xAC;
                }
                printf("past time = %d\n",GetTickCount() - dwStart);
            }
        }
        CloseComm();
    } while(0);
    return(0);
}
