;/*==========================================================================*\
;|                                                                            |
;| JTAGfunc.c                                                                 |
;|                                                                            |
;| JTAG Control Sequences for Erasing / Programming / Fuse Burning            |
;|----------------------------------------------------------------------------|
;| Project:              JTAG Functions                                       |
;| Developed using:      IAR Embedded Workbench 2.31C                         |
;|----------------------------------------------------------------------------|
;| Author:               FRGR                                                 |
;| Version:              1.2                                                  |
;| Initial Version:      04-17-02                                             |
;| Last Change:          08-29-02                                             |
;|----------------------------------------------------------------------------|
;| Version history:                                                           |
;| 1.0 04/02 FRGR        Initial version.                                     |
;| 1.1 04/02 ALB2        Formatting changes, added comments.                  |
;| 1.2 08/02 ALB2        Initial code release with Lit# SLAA149.              |
;|----------------------------------------------------------------------------|
;| Designed 2002 by Texas Instruments Germany                                 |
;\*==========================================================================*/


;/****************************************************************************/
;/* High level routines for accessing the target device via JTAG:            */
;/*                                                                          */
;/* From the following, the user is relieved from coding anything.           */
;/* To provide better understanding and clearness, some functionality is     */
;/* coded generously. (Code and speed optimization enhancements may          */
;/* be desired)                                                              */
;/****************************************************************************/

;//----------------------------------------------------------------------------
;/* Function for shifting a given 16-bit word into the JTAG data register
;   through TDI.
;   Arguments: word data (16-bit data, MSB first)
;   Result:    word (value is shifted out via TDO simultaneously)
;*/
;word DR_Shift16(word data) {
;r0:data
DR_Shift16:
	;// JTAG FSM state = Run-Test/Idle
		SetTMS				; TMS = 1
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
	;// JTAG FSM state = Select DR-Scan
		ClrTMS				; TMS = 0
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
	;// JTAG FSM state = Capture-DR
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
	;// JTAG FSM state = Shift-DR, Shift in TDI (16-bit)
		mov.b	#F_WORD,r1l		; 
		jsr.w	Shift			; 
	;// JTAG FSM state = Run-Test/Idle
		rts				; 

;//----------------------------------------------------------------------------
;/* Function for shifting a new instruction into the JTAG instruction 
;   register through TDI (MSB first, but with interchanged MSB - LSB, to
;   simply use the same shifting function, Shift(), as used in DR_Shift16).
;   Arguments: byte Instruction (8bit JTAG instruction, MSB first)
;   Result:    word TDOword (value shifted out from TDO = JTAG ID)
;*/
;word IR_Shift(byte instruction) {
;r0l:instaruction
IR_Shift:
	;// JTAG FSM state = Run-Test/Idle
		SetTMS				; TMS = 1
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
	;// JTAG FSM state = Select DR-Scan
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
	;// JTAG FSM state = Select IR-Scan
		ClrTMS				; TMS = 0
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
	;// JTAG FSM state = Capture-IR
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
	;// JTAG FSM state = Shift-IR, Shift in TDI (8-bit)
		mov.b	r0l,r0h			; 
		mov.b	#F_BYTE,r1l		; 
		jsr.w	Shift			; 
	;// JTAG FSM state = Run-Test/Idle
		mov.b	r0l,r0h			; 
		rts				; 

;//----------------------------------------------------------------------------
;/* Reset target JTAG interface and perform fuse-HW check.
;   Arguments: None
;   Result:    None
;*/
;void ResetTAP(void) {
ResetTAP:
		SetTDI				; TDI = 1
		SetTMS				; TMS = 1
		SetTCK				; TCK = 1
	;// Now fuse is checked, Reset JTAG FSM
		mov.b	#6,r0l			; 
ResetTAP05:
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
		sbjnz.b	#1,r0l,ResetTAP05	; 
	;// JTAG FSM is now in Test-Logic-Reset
		ClrTMS				; TMS = 0
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
	;// JTAG FSM is now in Run-Test/IDLE

	;// Perform fuse check
		ClrTMS				; TMS = 0
		SetTMS				; TMS = 1
		ClrTMS				; TMS = 0
		SetTMS				; TMS = 1
		rts				; 

;//----------------------------------------------------------------------------
;/* Function to execute a Power-Up Clear (PUC) using JTAG CNTRL SIG register
;   Arguments: None
;   Result:    word (STATUS_OK if JTAG ID is valid, STATUS_ERROR otherwise)
;*/
;word ExecutePUC(void) {
ExecutePUC:
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2C01h,r0		; // Apply Reset
		jsr.w	DR_Shift16		; 
		mov.w	#2401h,r0		; // Remove Reset
		jsr.w	DR_Shift16		; 
		ClrTCLK				; TCLK = 0
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.b	#IR_ADDR_CAPTURE,r0l	; 
		jsr.w	IR_Shift		; reference RET !!!
		push.w	r0			; save version
		SetTCLK				; TCLK = 1
	;// Disable Watchdog on target device
		push.w	a0			; 

		mov.b	#F_WORD,r0l		; 
		mov.w	#5A80h,r1		; data
		mov.w	#0120h,a0		; address
		jsr.w	WriteMem		; 
		pop.w	a0			; 

;see slaa149b
;		mov.b	#IR_DATA_16BIT,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#3FFFh,r0		; // JUMP $  instruction
;		jsr.w	DR_Shift16		; 
;		ClrTCLK				; TCLK = 0
;		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#2409h,r0		; // set HALT_JTAG bit
;		jsr.w	DR_Shift16		; 
;		SetTCLK				; TCLK = 1
;		ClrTCLK				; TCLK = 0
;		mov.b	#IR_CNTRL_SIG_16BIT,r0	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#2408h,r0		; // set to Write
;		jsr.w	DR_Shift16		; 
;		mov.b	#IR_ADDR_16BIT,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#0120h,r0		; // set Watchdog Control Register
;		jsr.w	DR_Shift16		; 
;		mov.b	#IR_DATA_TO_ADDR,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#5A80h,r0		; // write Watchdog Control Register
;		jsr.w	DR_Shift16		; 
;		SetTCLK				; TCLK = 1
;		ClrTCLK				; TCLK = 0
;		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#2401h,r0		; // set to read
;		jsr.w	DR_Shift16		; 
;		mov.b	#IR_ADDR_CAPTURE,r0l	; 
;		jsr.w	IR_Shift		; 
;		SetTCLK				; TCLK = 1

		pop.w	r1			; reload version
		mov.w	#STATUS_ERROR,r0	; 
		cmp.b	#JTAG_ID,r1l		; 
		jnz	ExecutePUC99		; 
		mov.w	#STATUS_OK,r0		; 
ExecutePUC99:
		rts				; 

;//----------------------------------------------------------------------------
;/* Function to take target device under JTAG control. Disables the target
;   watchdog. Sets the global DEVICE variable as read from the target device.
;   Arguments: None
;   Result:    word (STATUS_ERROR if fuse is blown, incorrect JTAG ID or
;                    synchronizing time-out; STATUS_OK otherwise)
;*/
;word GetDevice(void) {
GetDevice:
		xor.w	r0,r0			; // Preset DEVICE with "not a device"
		mov.w	r0,DEVICE		; 
		jsr.w	ResetTAP		; // Reset JTAG state machine, check fuse HW
		jsr.w	IsFuseBlown		; 
		cmp.w	#STATUS_OK,r0		; 
		jnz	GetDevice05		; 
	;// Stop here if fuse is already blown
		jmp.b	GetDevice99		; 
GetDevice05:
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2401h,r0		; // Set device into JTAG mode + read
		jsr.w	DR_Shift16		; 
		mov.b	#IR_CNTRL_SIG_CAPTURE,r0l	; 
		jsr.w	IR_Shift		; reference RET !!!
		cmp.b	#JTAG_ID,r0l		; 
		jz	GetDevice10		; 
		jmp.b	GetDevice99		; 
GetDevice10:
	;// Wait until CPU is synchronized, timeout after a limited # of attempts
		mov.b	#50,r1l			; 
GetDevice15:
		push.w	r1			; 
		mov.w	#0000h,r0		; 
		jsr.w	DR_Shift16		; reference RET !!!
		pop.w	r1			; 
		and.b	#02h,r0h		; 
		jnz	GetDevice20		; 
		sbjnz.b	#1,r1l,GetDevice15	; 
		jmp.b	GetDevice99		; // Timeout reached, return false
GetDevice20:
		mov.b	#F_WORD,r0l		; 
		mov.w	#0FF0h,r1		; // Get target device type
		jsr.w	ReadMem			; 
		xchg.b	r0l,r0h			; //(bytes are interchanged)
		mov.w	r0,DEVICE		; 
		jmp.w	ExecutePUC		; // Perform PUC, Includes
						; // target Watchdog disable.
GetDevice99:
		mov.w	#STATUS_ERROR,r0	; 
		rts				; 

;//----------------------------------------------------------------------------
;/* Function to set target CPU JTAG FSM into the instruction fetch state
;   Argument: None
;   Result:   word (STATUS_OK if instr. fetch was set, STATUS_ERROR otherwise)
;*/
;word SetInstrFetch(void) {
SetInstrFetch:
		mov.b	#IR_CNTRL_SIG_CAPTURE,r0l	; 
		jsr.w	IR_Shift		; 
	;// Wait until CPU is in instr. fetch state, timeout after limited attempts
		mov.b	#50,r1l			; 
SetInstrFetch05:
;;;		ClrTCLK				; TCLK = 0
;;;		SetTCLK				; TCLK = 1
		push.w	r1			; 
		mov.w	#00000h,r0		; 
		jsr.w	DR_Shift16		; reference RET !!!
		pop.w	r1			; 
		and.b	#80h,r0l		; //#80 ޸ނĂ
		jnz	SetInstrFetch10		; 
;see slaa149b
		ClrTCLK				; TCLK = 0
		SetTCLK				; TCLK = 1
		sbjnz.b	#1,r1l,SetInstrFetch05	; 
		mov.w	#STATUS_ERROR,r0	; 
		jmp.s	SetInstrFetch99		; 
SetInstrFetch10:
		mov.w	#STATUS_OK,r0		; 
SetInstrFetch99:
		rts				; 

;//----------------------------------------------------------------------------
;/* Load a given address into the target CPU's program counter (PC).
;   Argument: word Addr (destination address)
;   Result:   None
;*/
;void SetPC(word Addr) {
;r0:Addr
SetPC:
		push.w	r0			; save Addr
		jsr.w	SetInstrFetch		; // Set CPU into instruction fetch mode, TCLK=1
	;// Load PC with address
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#3401h,r0		; // CPU has control of RW & BYTE.
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#4030h,r0		; // "mov #addr,PC" instruction
		jsr.w	DR_Shift16		; 
;see slaa149b
		ClrTCLK				; TCLK = 0
		SetTCLK				; TCLK = 1
		pop.w	r0			; restore Addr
		jsr.w	DR_Shift16		; // Send addr value
		ClrTCLK				; TCLK = 0
		mov.b	#IR_ADDR_CAPTURE,r0l	; 
		jsr.w	IR_Shift		; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0 // Now the PC should be on Addr
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2401h,r0		; // JTAG has control of RW & BYTE.
		jsr.w	DR_Shift16		; 
		rts				; 

;//----------------------------------------------------------------------------
;/* Function to set the CPU into a controlled stop state
;*/
;void HaltCPU(void) {
HaltCPU:
		jsr.w	SetInstrFetch		; // Set CPU into instruction fetch mode
;see slaa149b
;;;		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
;;;		jsr.w	IR_Shift		; 
;;;		mov.w	#2401h,r0		; 
;;;		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#3FFFh,r0		; // Send JMP $ instruction
		jsr.w	DR_Shift16		; 
;see slaa149b
;;;		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2409h,r0		; // Set JTAG_HALT bit
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
		rts				; 

;//----------------------------------------------------------------------------
;/* Function to release the target CPU from the controlled stop state
;*/
;void ReleaseCPU(void) {
ReleaseCPU:
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2401h,r0		; // Clear the HALT_JTAG bit
		jsr.w	DR_Shift16		; 
		mov.b	#IR_ADDR_CAPTURE,r0l	; 
		jsr.w	IR_Shift		; 
		SetTCLK				; TCLK = 1
		rts				; 

;//----------------------------------------------------------------------------
;/* Function to release the target device from JTAG control
;   Argument: word Addr (0xFFFE: Perform Reset, means Load Reset Vector into PC,
;                        otherwise: Load Addr into PC)
;   Result:   None
;*/
;void ReleaseDevice(word Addr) {
;r0:Addr
ReleaseDevice:
		cmp.w	#V_RESET,r0		; 
		jnz	ReleaseDevice05		; 
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2C01h,r0		; // Perform a reset
		jsr.w	DR_Shift16		; 
		mov.w	#2401h,r0		; 
		jsr.w	DR_Shift16		; 
		jmp.s	ReleaseDevice10		; 
ReleaseDevice05:
		jsr.w	SetPC			; // Set target CPU's PC
ReleaseDevice10:
		mov.b	#IR_CNTRL_SIG_RELEASE,r0l	; 
		jsr.w	IR_Shift		; 
		rts				; 

;//----------------------------------------------------------------------------
;/* This function compares the computed PSA (Pseudo Signature Analysis) value
;   to the PSA value shifted out from the target device.
;   It is used for very fast data block write or erasure verification.
;   Arguments: word StartAddr (Start address of data block to be checked)
;              word Length (Number of words within data block)
;              word *DataArray (Pointer to array with the data, 0 for Erase Check)
;   Result:    word (STATUS_OK if comparison was successful, STATUS_ERROR otherwise)
;*/
;// 삹
;word VerifyPSA(word StartAddr,word Length,word *DataArray) {
;r0:StartAddr,r1:Length,a0:DataArray
VerifyPSA:
	.IF	JTYP==0
		push.w	r1			; save Length
		add.w	#-2,r0			; 
		push.w	r0			; save StartAddr
		jsr.w	ExecutePUC		; //װ肵Ȃ
;slaa149b if
;		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#2401h,r1		; 
;		jsr.w	DR_Shift16		; 
;		jsr.w	SetInstrFetch		; 
;		mov.b	#IR_DATA_16BIT,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#4030h,r0		; 
;		jsr.w	DR_Shift16		; 
;		SetTCLK				; TCLK = 1
;		ClrTCLK				; TCLK = 0
;		pop.w	r0			; restore StartAddr
;		push.w	r0			; save PSA_CRC
;		jsr.w	DR_Shift16		; 
;		SetTCLK				; TCLK = 1
;		ClrTCLK				; TCLK = 0
;		SetTCLK				; TCLK = 1
;		ClrTCLK				; TCLK = 0
;		SetTCLK				; TCLK = 1
;		ClrTCLK				; TCLK = 0
;		mov.b	#IR_ADDR_CAPTURE,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#0000h,r0		; 
;		jsr.w	DR_Shift16		; 
;slaa149b else
		pop.w	r0			; restore StartAddr
		push.w	r0			; save PSA_CRC
		jsr.w	SetPC			; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
;slaa149b
		mov.b	#IR_DATA_PSA,r0l	; 
		jsr.w	IR_Shift		; 
		pop.w	r1			; restore PSA_CRC
		pop.w	r0			; restore Length
VerifyPSA05:
	;// Calculate the PSA (Pseudo Signature Analysis) value
		or.b	r1h,r1h			; if((PSA_CRC & 0x8000) == 0x8000) {
		fclr	c			; 
		jpz	VerifyPSA10		; 
		xor.w	#0805h,r1		; PSA_CRC ^= POLY(0x0805)
		fset	c			; PSA_CRC <<= 1,PSA_CRC |= 1
VerifyPSA10:
		rolc.w	r1			; PSA_CRC <<= 1,PSA_CRC |= 1(0)
	;// if pointer is 0 then use erase check mask, otherwise data
		or.w	a0,a0			; 
		jz	VerifyPSA15		; 
		xor.b	[a0],r1l		; 
		xor.b	1[a0],r1h		; 
		add.w	#2,a0			; 
		jmp.s	VerifyPSA20		; 
VerifyPSA15:
		xor.w	#0FFFFh,r1		; PSA_CRC ^= 0xFFFF
VerifyPSA20:
	;// Clock through the PSA
		SetTCLK				; TCLK = 1
;slaa149b
		ClrTCLK				; TCLK = 0

		SetTMS				; TMS = 1
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Select DR scan

		ClrTMS				; TMS = 0
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Capture DR

		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Shift DR

		SetTMS				; TMS = 1
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Exit DR

		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Update DR

		ClrTMS				; TMS = 0
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Run Test/IDLE

;;;		ClrTCLK				; TCLK = 0

		sbjnz.w	#1,r0,VerifyPSA05	; 
		push.w	r1			; save PSA_CRC
		mov.b	#IR_SHIFT_OUT_PSA,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0000h,r0		; // Read out the PSA value
		jsr.w	DR_Shift16		; reference RET !!!
		SetTCLK				; TCLK = 1
;slaa149b
		push.w	r0			; 
		jsr.w	ExecutePUC		; 
		pop.w	r0			; 
		pop.w	r1			; restore PSA_CRC

		mov.w	a0,r0			; for debug
		cmp.w	r0,r1			; (TDOword == PSA_CRC)?  STATUS_OK : STATUS_ERROR
		jz	VerifyPSA25		; 
		mov.w	#STATUS_ERROR,r0	; 
		jmp.s	VerifyPSA99		; 
VerifyPSA25:
		mov.w	#STATUS_OK,r0		; 
VerifyPSA99:
		rts				; 
	.ELSE
		push.w	r1			; save Length
		add.w	#-2,r0			; 
		push.w	r0			; save StartAddr

	;// CRCvZ
VerifyPSA05:
	;// Calculate the PSA (Pseudo Signature Analysis) value
		or.b	r0h,r0h			; if((PSA_CRC & 0x8000) == 0x8000) {
		fclr	c			; 
		jpz	VerifyPSA10		; 
		xor.w	#0805h,r0		; PSA_CRC ^= POLY(0x0805)
		fset	c			; PSA_CRC <<= 1,PSA_CRC |= 1
VerifyPSA10:
		rolc.w	r0			; PSA_CRC <<= 1,PSA_CRC |= 1(0)
	;// if pointer is 0 then use erase check mask, otherwise data
		or.w	a0,a0			; 
		jz	VerifyPSA15		; 
		xor.b	[a0],r0l		; 
		xor.b	1[a0],r0h		; 
		add.w	#2,a0			; 
		jmp.s	VerifyPSA20		; 
VerifyPSA15:
		xor.w	#0FFFFh,r0		; PSA_CRC ^= 0xFFFF
VerifyPSA20:
		sbjnz.w	#1,r1,VerifyPSA05	; 
		mov.w	r0,a0			; save PSA_CRC

		jsr.w	ExecutePUC		; //װ肵Ȃ
;slaa149b if
;		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#2401h,r1		; 
;		jsr.w	DR_Shift16		; 
;		jsr.w	SetInstrFetch		; 
;		mov.b	#IR_DATA_16BIT,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#4030h,r0		; 
;		jsr.w	DR_Shift16		; 
;		SetTCLK				; TCLK = 1
;		ClrTCLK				; TCLK = 0
;		pop.w	r0			; restore StartAddr
;		push.w	r0			; save PSA_CRC
;		jsr.w	DR_Shift16		; 
;		SetTCLK				; TCLK = 1
;		ClrTCLK				; TCLK = 0
;		SetTCLK				; TCLK = 1
;		ClrTCLK				; TCLK = 0
;		SetTCLK				; TCLK = 1
;		ClrTCLK				; TCLK = 0
;		mov.b	#IR_ADDR_CAPTURE,r0l	; 
;		jsr.w	IR_Shift		; 
;		mov.w	#0000h,r0		; 
;		jsr.w	DR_Shift16		; 
;slaa149b else
		pop.w	r0			; restore StartAddr
		jsr.w	SetPC			; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
;slaa149b
		mov.b	#IR_DATA_PSA,r0l	; 
		jsr.w	IR_Shift		; 
		pop.w	r0			; restore Length
VerifyPSA25:
	;// Clock through the PSA
		SetTCLK				; TCLK = 1
;slaa149b
		ClrTCLK				; TCLK = 0

		SetTMS				; TMS = 1
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Select DR scan

		ClrTMS				; TMS = 0
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Capture DR

		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Shift DR

		SetTMS				; TMS = 1
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Exit DR

		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Update DR

		ClrTMS				; TMS = 0
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1 // Run Test/IDLE

;;;		ClrTCLK				; TCLK = 0

		sbjnz.w	#1,r0,VerifyPSA25	; 

		mov.b	#IR_SHIFT_OUT_PSA,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0000h,r0		; // Read out the PSA value
		jsr.w	DR_Shift16		; reference RET !!!
		SetTCLK				; TCLK = 1
;slaa149b
		push.w	r0			; 
		jsr.w	ExecutePUC		; 
		pop.w	r0			; 

		mov.w	a0,r1			; restore PSA_CRC

		mov.w	a0,r0			; for debug
		cmp.w	r0,r1			; (TDOword == PSA_CRC)?  STATUS_OK : STATUS_ERROR
		jz	VerifyPSA30		; 
		mov.w	#STATUS_ERROR,r0	; 
		jmp.s	VerifyPSA99		; 
VerifyPSA30:
		mov.w	#STATUS_OK,r0		; 
VerifyPSA99:
		rts				; 
	.ENDIF

;//----------------------------------------------------------------------------
;/* This function writes one byte/word at a given address ( <0xA00) 
;   Arguments: word Format (F_BYTE or F_WORD)
;              word Addr (Address of data to be written)
;              word Data (shifted data)  
;   Result:    None
;*/
;r0l:Format,a0:Addr,r1:Data
;void WriteMem(word Format,word Addr,word Data) {
WriteMem:
		push.w	r1			; save Data
		push.w	r0			; save Format
		jsr.w	HaltCPU			; 
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		pop.w	r1			; restore Format
		mov.w	#2408h,r0		; word write
		cmp.b	#F_WORD,r1l		; 
		jz	WriteMem05		; 
		mov.w	#2418h,r0		; byte write
WriteMem05:
		jsr.w	DR_Shift16		; ߂Ȃ֐̏ꍇAPUSH/POP mF
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	a0,r0			; // Set addr
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		pop.w	r0			; restore Data
		jsr.w	DR_Shift16		; // Shift in 16 bits
		SetTCLK				; TCLK = 1
		jsr.w	ReleaseCPU		; 
		rts				; 

;//----------------------------------------------------------------------------
;/* This function writes an array of words into teh target memory.
;   Arguments: word StartAddr (Start address of target memory)
;              word Length (Number of words to be programmed)
;              word *DataArray (Pointer to array with the data)
;   Result:    None
;*/
;r0:StartAddr,r1:Length,a0:DataArray
;void WriteMemQuick(word StartAddr,word Length,word *DataArray) {
WriteMemQuick:
	;// Initialize writing:
		push.w	r1			; save Length
		add.w	#-4,r0			; 
		jsr.w	SetPC			; 
		jsr.w	HaltCPU			; 
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2408h,r0		; // Set RW to write
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_QUICK,r0l	; 
		jsr.w	IR_Shift		; 
		pop.w	r0			; restore Length
WriteMemQuick05:
		push.w	r0			; 
		mov.b	[a0],r0l		; // Shift in the write data
		mov.b	1[a0],r0h		; 
		add.w	#2,a0			; 
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0 // Increment PC by 2
		pop.w	r0			; 
		sbjnz.w	#1,r0,WriteMemQuick05	; 
;!!!
;;;		SetTCLK				; TCLK = 1
		jsr.w	ReleaseCPU		; 
		rts				; 

;//----------------------------------------------------------------------------
;/* This function reads one byte/word from a given address in memory
;   Arguments: word Format (F_BYTE or F_WORD)
;              word Addr (address of memory)
;   Result:    word (content of the addressed memory location)
;*/
;r0l:Format,r1:Addr
;return r0:
;word ReadMem(word Format,word Addr) {
ReadMem:
		push.w	r2			; 
		push.w	r1			; save Addr
		xor.b	r0h,r0h			; 
		mov.w	r0,r2			; save Format
		jsr.w	HaltCPU			; 
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2409h,r0		; word read
		cmp.w	#F_WORD,r2		; 
		jz	ReadMem05		; 
		mov.w	#2419h,r0		; byte read
ReadMem05:
		jsr.w	DR_Shift16		; 
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		pop.w	r0			; // Set address
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.w	#0000h,r0		; // Shift out 16 bits
		jsr.w	DR_Shift16		; reference RET !!!
		cmp.w	#F_WORD,r2		; 
		jz	ReadMem10		; 
		mov.b	r0l,r0h			; 
ReadMem10:
		mov.w	r0,r2			; 
		SetTCLK				; TCLK = 1
		jsr.w	ReleaseCPU		; 
		mov.w	r2,r0			; 
		pop.w	r2			; 
		rts				; 

;//----------------------------------------------------------------------------
;/* This function reads an array of words from a memory.
;   Arguments: word StartAddr (Start address of memory to be read)
;              word Length (Number of words to be read)
;              word *DataArray (Pointer to array for the data)
;   Result:    None
;*/
;// 삹
;r0:StartAddr,r1:Length,a0:DataArray
;void ReadMemQuick(word StartAddr, word Length, word *DataArray) {
ReadMemQuick:
	;// Initialize reading:
		push.w	r1			; save Length
		add.w	#-4,r0			; 
		jsr.w	SetPC			; 
		jsr.w	HaltCPU			; 
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2409h,r0		; // Set RW to read
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_QUICK,r0l	; 
		jsr.w	IR_Shift		; 
		pop.w	r0			; restore Length
ReadMemQuick05:
		push.w	r0			; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.w	#0000h,r0		; // Shift out the data
		jsr.w	DR_Shift16		; reference RET !!!
		mov.b	r0l,[a0]		; // from the target.
		mov.b	r0h,1[a0]		; 
		add.w	#2,a0			; 
		pop.w	r0			; 
		sbjnz.w	#1,r0,ReadMemQuick05	; 
;!!!
;;;		SetTCLK				; TCLK = 1
		jsr.w	ReleaseCPU		; 
		rts				; 

;//----------------------------------------------------------------------------
;/* This function performs a mass erase (with and w/o info memory) or a segment
;   erase of a FLASH module specified by the given mode and address.
;   Large memory devices get additional mass erase operations to meet the spec.
;   Arguments: word Mode (could be ERASE_MASS or ERASE_MAIN or ERASE_SGMT)
;              word Addr (any address within the selected segment)
;   Result:    None
;   Remark:    Could be extended with erase check via PSA.
;*/
;void EraseFLASH(word EraseMode,word EraseAddr) {
;SP+0:
;  +1:loopcount
;  +2:StrobeAmount
;  +3:
;  +4:EraseMode
;  +5:
;r0:EraseMode,a0:EraseAddr
EraseFLASH:
		push.w	r0			; // mode
		mov.w	#4820,r1		; // default for Segment Erase
		push.w	r1			; 
		mov.w	#1,r1			; // erase cycle repeating for Mass Erase
		push.w	r1			; 
		cmp.w	#ERASE_MASS,r0		; 
		jz	EraseFLASH05		; 
		cmp.w	#ERASE_MAIN,r0		; 
		jnz	EraseFLASH10		; 
EraseFLASH05:
		mov.w	#5300,r0		; // Larger Flash memories require
		mov.w	r0,2[sp]		; 
		mov.w	#19,r0			; // additional cycles for erase.
		mov.w	r0,0[sp]		; 
EraseFLASH10:
		jsr.w	HaltCPU			; 

EraseFLASH15:					; // loop
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2408h,r0		; // set RW to write
		jsr.w	DR_Shift16		; 
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0128h,r0		; // FCTL1 address
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	4[sp],r0		; // Enable erase mode
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#012Ah,r0		; // FCTL2 address
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0A540h,r0		; // MCLK is source, DIV=1
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#012Ch,r0		; // FCTL3 address
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0A500h,r0		; // Clear FCTL3
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	a0,r0			; // Set erase address
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#55AAh,r0		; // Dummy write to start erase
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2409h,r0		; // Set RW to read
		jsr.w	DR_Shift16		; 
		mov.w	2[sp],r0		; StrobeAmount
		jsr.w	TCLKstrobes		;// Provide TCLKs
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2408h,r0		; // Set RW to write
		jsr.w	DR_Shift16		; 
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0128h,r0		; // FCTL1 address
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0A500h,r0		; // Disable erase
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
		mov.w	0[sp],r0		; 
		add.w	#-1,r0			; 
		mov.w	r0,0[sp]		; 
		jnz	EraseFLASH15		; 

;//EraseFLASH20:
;//slaa149b
		ClrTCLK				; 
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#012Ch,r0		; // FCTL3 address
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0A500h,r0		; // Disable FLASH Write Access
		jsr.w	DR_Shift16		; 
		SetTCLK				; 
;

		jsr.w	ReleaseCPU		; 
		add.w	#6,sp			; 
		rts				; 

;//----------------------------------------------------------------------------
;/* This function performs an Erase Check over the given memory range
;   Arguments: word StartAddr (Start address of memory to be checked)
;              word Length (Number of words to be checked)
;   Result:    word (STATUS_OK if erase check was successful, STATUS_ERROR otherwise)
;*/
;word EraseCheck(word StartAddr,word Length) {
;r0:StartAddr,r1:Length
EraseCheck:
		mov.w	#0,a0			; 
		jmp.w	VerifyPSA		; 

;//----------------------------------------------------------------------------
;/* This function performs a Verification over the given memory range
;   Arguments: word StartAddr (Start address of memory to be verified)
;              word Length (Number of words to be verified)
;              word *DataArray (Pointer to array with the data)
;   Result:    word (STATUS_OK if verification was successful, STATUS_ERROR otherwise)
;*/
;word VerifyMem(word StartAddr,word Length, word *DataArray) {
;r0:StartAddr,r1:Length,a0:DataArray
VerifyMem:
		jmp.w	VerifyPSA		; 

;//----------------------------------------------------------------------------
;/* This function programs/verifies an array of words into an FLASH by
;   using the FLASH controller. The JTAG FLASH register isn't needed.
;   Arguments: word StartAddr (Start address of FLASH memory)
;              word Length (Number of words to be programmed)
;              word *DataArray (Pointer to array with the data)
;   Result:    None
;*/
;void WriteFLASH(word StartAddr,word Length,word *DataArray) {
;SP+0:Length
;  +1:
;  +2:StartAddr
;  +3:
;r0:StartAddr,r1:Length,a0:DataArray
WriteFLASH:
		push.w	r0			; save StartAddr
		push.w	r1			; save Length
		jsr.w	HaltCPU			; 
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2408h,r0		; // Set RW to write
		jsr.w	DR_Shift16		; 
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0128h,r0		; // FCTL1 register
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0A540h,r0		; // Enable FLASH write
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#012Ah,r0		; // FCTL2 register
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0A540h,r0		; // Select MCLK as source, DIV=1
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#012Ch,r0		; // FCTL3 register
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0A500h,r0		; // Clear FCTL3 register
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
;slaa149b
;;;		ClrTCLK				; TCLK = 0
WriteFLASH05:
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2408h,r0		; // Set RW to write
		jsr.w	DR_Shift16		; 
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	2[sp],r0		; // Set address
		mov.w	r0,r1			; addr += 2
		add.w	#2,r1			; 
		mov.w	r1,2[sp]		; 
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.b	[a0],r0l		; // Set data
		mov.b	1[a0],r0h		; 
		add.w	#2,a0			; 
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1
		ClrTCLK				; TCLK = 0
		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2409h,r0		; // Set RW to read
		jsr.w	DR_Shift16		; 
		mov.w	#35,r0			; 
		jsr.w	TCLKstrobes		; // Provide TCLKs, min. 33 for F149 and F449 
		mov.w	0[sp],r0		; Length
		add.w	#-1,r0			; 
		mov.w	r0,0[sp]		; 
		jnz	WriteFLASH05		; 

		mov.b	#IR_CNTRL_SIG_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#2408h,r0		; // Set RW to write
		jsr.w	DR_Shift16		; 
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0128h,r0		; // FCTL1 register
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0A500h,r0		; // Disable FLASH write
		jsr.w	DR_Shift16		; 
		SetTCLK				; TCLK = 1

;//slaa149b(TNK)
		ClrTCLK				; 
		mov.b	#IR_ADDR_16BIT,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#012Ch,r0		; // FCTL3 address
		jsr.w	DR_Shift16		; 
		mov.b	#IR_DATA_TO_ADDR,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0A500h,r0		; // Disable FLASH Write Access
		jsr.w	DR_Shift16		; 
		SetTCLK				; 
;
		jsr.w	ReleaseCPU		; 
		add.w	#4,sp			; 
		rts				; 

;//------------------------------------------------------------------------
;/* This function checks if the JTAG access security fuse is blown.
;   Arguments: None
;   Result:    word (STATUS_OK if fuse is blown, STATUS_ERROR otherwise)
;*/
;word IsFuseBlown(void) {
IsFuseBlown:
	;//  First trial could be wrong
		mov.b	#3,r1l			; 
IsFuseBlown05:
		push.w	r1			; 
		mov.b	#IR_CNTRL_SIG_CAPTURE,r0l	; 
		jsr.w	IR_Shift		; 
		mov.w	#0AAAAh,r0		; 
		jsr.w	DR_Shift16		; reference RET !!!
		pop.w	r1			; 
		cmp.w	#5555h,r0		; 
		jz	IsFuseBlown10		; 
		sbjnz.b	#1,r1l,IsFuseBlown05	; 
		mov.w	#STATUS_ERROR,r0	; // fuse is not blown
		jmp.s	IsFuseBlown99		; 
IsFuseBlown10:
		mov.w	#STATUS_OK,r0		; // Fuse is blown
IsFuseBlown99:
		rts				; 

;/****************************************************************************/
;/*                         END OF SOURCE FILE                               */
;/****************************************************************************/
