;/*==========================================================================*\
;|                                                                            |
;| LowLevelFunc.c                                                             |
;|                                                                            |
;| Low Level Functions regarding user's Hardware                              |
;|----------------------------------------------------------------------------|
;| Project:              MSP430 Replicator                                    |
;| Developed using:      IAR Embedded Workbench 2.31C                         |
;|----------------------------------------------------------------------------|
;| Author:               FRGR                                                 |
;| Version:              1.3                                                  |
;| Initial Version:      04-17-02                                             |
;| Last Change:          08-29-02                                             |
;|----------------------------------------------------------------------------|
;| Version history:                                                           |
;| 1.0 04/02 FRGR        Initial version.                                     |
;| 1.1 04/02 FRGR        Included SPI mode to speed up shifting function by 2.|
;| 1.2 06/02 ALB2        Formatting changes, added comments.                  |
;| 1.3 08/02 ALB2        Initial code release with Lit# SLAA149.              |
;|----------------------------------------------------------------------------|
;| Designed 2002 by Texas Instruments Germany                                 |
;\*==========================================================================*/


;/****************************************************************************/
;/* Function declarations which have to be programmed by the user for use    */
;/* with hosts other than the MSP430F149.                                    */
;/*                                                                          */
;/* The following MSP430F149-specific code can be used as a reference as to  */
;/* how to implement the required JTAG communication on additional hosts.    */
;/****************************************************************************/

;//----------------------------------------------------------------------------
;/*  Shift a value into TDI (MSB first) and simultaneously shift out a value
;    from TDO (MSB first).
;    Note:
;    Arguments: word Format (number of bits shifted, 8 (F_BYTE) or 16 (F_WORD))
;               word Data (data to be shifted into TDI)
;    Result:    word (scanned TDO value)
;*/
;word Shift(word Format,word Data) {
;r0:Data,r1l:Format
;called from DR_Shift16, IR_Shift
;r0:return,r1:unknown
Shift:
		mov.b	#8,r1h			; 
		cmp.b	#F_WORD,r1l		; 
		jnz	Shift05			; 
		shl.b	#1,r1h			; 
Shift05:
		StoreTCLK	r1l		; // Store TCLK state;
Shift10:
		shl.w	#1,r0			; 
		bmc	TDI			; 
		cmp.b	#1,r1h			; // Last bit requires TMS=1
		jnz	Shift15			; 
		SetTMS				; TMS = 1
Shift15:
		ClrTCK				; TCK = 0

		SetTCK				; TCK = 1

		btst	TDO			; // TDO could be any port pin
		bmc	0,r0			; 

		sbjnz.b	#1,r1h,Shift10		; 
	;// common exit
		RestoreTCLK	r1		; // restore TCLK state

	;// JTAG FSM = Exit-DR
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
	;// JTAG FSM = Update-DR
		ClrTMS				; TMS = 0
		ClrTCK				; TCK = 0
		SetTCK				; TCK = 1
	;// JTAG FSM = Run-Test/Idle
		rts				; 

;/*----------------------------------------------------------------------------
;   Delay function (resolution is 1 ms)
;   User knows target frequency, instruction cycles, C implementation.
;   Arguments: word millisec (number of ms, max number is 0xFFFF)     
;*/
;void Delay(word millisec)           // Inner loop generates 1 ms
Delay:
		bclr	txck0			; XĥI(f1)
		bclr	txck1			;         
		mov.b	#00000000b,txmr		; 
			;||||||++--- txmod - 00:ϥӰ
			;||||||              01:ٽoӰ
			;||||||              10:ĥĥӰ
			;||||||              11:ٽӰ
			;|||||+----- R0EDG -    INT1#/CNTR0Mɐؑޯ
			;||||+------ TXS   -  0:Ē~1:ĊJn
			;|||+------- TXOCNT     P3_7/CNTR0#Iޯ
			;||+-------- TXMOD2-  0:ٽӰވȊO
			;||                   1:ٽӰ
			;|+--------- TXEDG      Lޔ׸
			;+---------- TXUND      Xް۰׸
		mov.b	#199,prex		; 0.05us~200=10us
		mov.b	#99,tx			; 10us~100=1000us(1ms)
		bset	txs			; 
Delay05:
		btstc	txund			; 
		jz	Delay05			; 
		sbjnz.w	#1,r0,Delay05		; 
		bclr	txs			; 
		rts				; 

;/*---------------------------------------------------------------------------
;   This function generates Amount strobes with the Flash Timing Generator
;   Frequency fFTG = 257..476kHz (t = 3.9..2.1us).
;   User knows target frequency, instruction cycles, C implementation.
;   Arguments: word Amount (number of strobes to be generated)
;*/
;void TCLKstrobes(word Amount)
TCLKstrobes:
		bset	TCLK			; 
		add.w	#-1,r0			; update amount
		jmp	TCLKstrobes05		; 
TCLKstrobes05:
		jmp	TCLKstrobes10		; 
TCLKstrobes10:
		jmp	TCLKstrobes15		; 
;;;		nop				; 
;;;		nop				; 
TCLKstrobes15:
		bclr	TCLK			; 
		jmp	TCLKstrobes20		; 
TCLKstrobes20:
		jmp	TCLKstrobes25		; 
TCLKstrobes25:
		jnz	TCLKstrobes		; 
		rts				; 

;/*----------------------------------------------------------------------------
;   This function controls the status LEDs depending on the status
;   argument. It stops program in error case.
;   Arguments: word status (4 stati possible for 2 LEDs)
;              word index (additional number for detailed diagnostics or
;                          watch variable during debugging phase) 
;*/
;void ShowStatus(word status,word index) {
;r0:status,r1:index
ShowStatus:
		bset	LEDGREEN		; LED OFF
		bset	LEDRED			; LED OFF
		cmp.w	#STATUS_ERROR,r0	; 
		jnz	ShowStatus05		; 
		bclr	LEDRED			; LED ON
		jmp.b	ShowStatus99		; 
ShowStatus05:
		cmp.w	#STATUS_ACTIVE,r0	; 
		jnz	ShowStatus10		; 
		bclr	LEDRED			; LED ON
		jmp.s	ShowStatus15		; 
ShowStatus10:
		cmp.w	#STATUS_OK,r0		; 
		jnz	ShowStatus99		; 
ShowStatus15:
		bclr	LEDGREEN		; LED ON
ShowStatus99:
		rts				; 

;/****************************************************************************/
;/*                         END OF SOURCE FILE                               */
;/****************************************************************************/
